<%
' AccessKeys CLASS (PHP 4)
' ========================
'
' Author: Gez Lemon (http://juicystudio.com)
' Usage:
' objAK = New AccessKeys
'
' AccessKey Methods
' ==================
' 
' The following lists the methods that can be used with the AccessKeys Class:
' 
' 
' addAccessitem Method
' ====================
' 
' The addAccessitem method allows a single item to be added to the collection 
' of items that may be assigned an accesskey.
' 
' @parameter String strItem (The name of the item to be added to the collection)
' @parameter String strValue (The value for the item)
' @parameter String strDefault (The suggested value for the access key, or Null for no suggestion)
' 
' Examples:
' objAK.addAccessitem "Meat", "meat.asp", Null
' objAK.addAccessitem "Fish", "fish.asp", "9"
' 
' 
' removeAccessItem Method
' =======================
' 
' The removeAccessitem method removes an item from the collection of items that 
' may be assigned an access key.
' 
' @parameter String strItem (The name of the item to be removed from the collection)
' 
' Example:
' objAK.removeAccessitem "Meat"
' 
' 
' isValid Method
' ==============
' 
' The isValid method returns a Boolean value indicating whether or not the values 
' assigned were valid.
' 
' @parameter none
' 
' Example:
' If Not objAK.isValid Then
' 	Response.Write objAK.getErrors
' End If
' 
' 
' getErrors Method
' ================
' 
' The getErrors method returns a string containing errors from form submissions, 
' or an empty string if there are no errors. The getErrors method should be used 
' with the isValid method.
' 
' @parameter none
' 
' Example:
' If Not objAK.isValid Then
' 	Response.Write objAK.getErrors
' End If
' 
' 
' setCookies Method
' =================
' 
' The setCookies method returns a Boolean value indicating whether or not the
' values were valid accesskey values. If the values are inappropriate, isValid
' returns False, and getErrors can be used to retrieve the errors.
' 
' @parameter none
' 
' Example:
' 
' objAK.setCookies
' 
' 
' removeAllBindings Method
' ========================
' 
' The removeAllBindings method removes all access keys defined for the collection.
' 
' @parameter none
' 
' Example:
' 
' objAK.removeAllBindings
' 
' 
' displayOptions Method
' =====================
' 
' The displayOptions method displays a definition list where the definition terms 
' are the names in the collection, and the definitions contain the assigned key, 
' and suggested key if available.
' 
' @parameter String strID (parameter to assign an id attribute to the definition list, Null if no parameter to be used)
' 
' Examples:
' objAK.displayOptions Null
' objAK.displayOptions "someid"
' 
' 
' generateForm Method
' ===================
' 
' The generateForm method generates form sections based on the names provided 
' in the collection. The method takes a parameter to indicate whether the 
' generated markup is XHTML.
' 
' @parameter Boolean bXHTML (Indicates markup format; True = XHTML, False = HTML)
' 
' Examples:
' 
' The following generates the form markup for HTML:
' 
' objAK.generateForm False
' 
' The next example generates the form markup for XHTML:
' 
' objAK.generateForm True
' 
' 
' useDefaults Method
' ==================
' 
' The useDefault method assigns all of the values that have been suggested as 
' default keys by the author, if any have been provided.
' 
' @parameter none
' 
' Example:
' 
' objAK.useDefaults
' 
' 
' getAccessitem Method
' ====================
' 
' The getAccessitem method returns the markup for a particular item, including 
' the accesskey if one has been defined. If the item doesn't exist in the collection, 
' the method returns an empty string. The accesskey can be ignored using the 
' bKey parameter, in which case, just the markup will be returned. As access keys
' may be assigned to links, labels, or form controls, a parameter allows
' the markup to return label or link tags.
' 
' @parameter String strItem (The item to be returned)
' @parameter Boolean bKey (Indicates whether the accesskey value should be included)
' @parameter String strElement (Element to be returned; either "link" or "label")
' 
' Examples:
' 
' An example of how this might be used in a navigation list
' 
' Response.Write objAK.getAccessitem("Home", True, "link")
' 
' An example of how this might be used with a form element
' 
' Response.Write objAK.getAccessitem("Search", true, "label")
' 
' 
Class AccessKeys
	Private dtExpires
	Private bValid
	Private strError
	Private objBindings
	Private objRE

	' Class events
	Private Sub Class_Initialize()
		Set objBindings = server.CreateObject("Scripting.Dictionary")
		Set objRE = New RegExp

		objRE.Global = True

		objBindings.CompareMode = 1
		dtExpires = DateAdd("yyyy", 1, Date())
		bValid = True
  	End Sub

	Private Sub Class_Terminate()
		Set objBindings = Nothing
		Set objRE = Nothing
	End Sub

	' Class properties
	Public Property Get isValid()
		isValid = bValid
	End Property

	Public Property Get getErrors()
		getErrors = strError
	End Property

	' Class methods
	Public Sub addAccessitem(strItem, strValue, strDefault)
		If Not IsNull(strDefault) And Trim(strDefault) <> "" Then 
			objBindings.Add Trim(strItem), Array(Trim(strValue), Trim(strDefault))
		Else
			objBindings.Add Trim(strItem), Trim(strValue)
		End If
	End Sub

	Public Sub removeAccessitem(strItem)
		objBindings.Remove strItem
	End Sub

	Public Function setCookies()
		Dim arKeys()
		Dim strFind, strAccess, strKey, strName
		Dim iCounter, iInner, iOuter

		bValid = True

		strError = "<div id=""jserror"">" & vbCrLf
		strError = strError & "<h2>Submission Error</h2>" & vbCrLf
		strError = strError & "<p>Please review the following</p>" & vbCrLf
		strError = strError & "<ul>" & vbCrLf
		
		iCounter = 0

		For Each strName In objBindings.Keys
			objRE.Pattern = "\s*"
			strFind = LCase(objRE.Replace(strName, ""))
			strAccess = Trim(Request.Form(strFind))
			strKey = Trim(Request.Form(strFind & "binding"))

			If Len(strAccess) > 0 Then
				objRE.Pattern = "[A-Za-z0-9]"
				If Not objRE.Test(strKey) Or Len(strKey) <> 1 Then
					bValid = False
					strError = strError & vbTab & "<li>Please provide a single alphanumeric character for " & strName & "</li>" & vbCrLf
				Else
					iCounter = iCounter + 1
					ReDim Preserve arKeys(iCounter)
					arKeys(iCounter-1) = strKey
				End If
			End If
		Next
		
		For iInner=0 To iCounter-1
			For iOuter=iInner+1 To iCounter-1
				If arKeys(iInner) = arKeys(iOuter) And Len(arKeys(iInner)) > 0 Then
					bValid = False
					strError = strError & vbTab & "<li>Please ensure the access key is unique. The value " & arKeys(iInner) & " was used more than once</li>" & vbCrLf
				End If
			Next
		Next

		If Not bValid Then
			strError = strError & "</ul>" & vbCrLf
			strError = strError & "</div>" & vbCrLf
		Else
			strError = ""
			objRE.Pattern = "\s*"

			For Each strName In objBindings.Keys
				strName = LCase(objRE.Replace(strName, ""))
				strAccess = Trim(Request.Form(strName))
				strKey = Trim(Request.Form(strName & "binding"))

				If Len(strAccess) > 0 Then
					Response.Cookies(strName) = strKey
				Else
					Response.Cookies(strName) = ""
				End If
				Response.Cookies(strName).Expires = dtExpires
			Next
		End If
		
		setCookies = bValid
	End Function

	Public Sub removeAllBindings()
		Dim strName

		objRE.Pattern = "\s*"

		For Each strName In objBindings.Keys
			strName = LCase(objRE.Replace(strName, ""))
			Response.Cookies(strName) = ""
			Response.Cookies(strName).Expires = dtExpires
		Next
	End Sub

	Public Sub displayOptions(strID)
		Dim strName, strKey, strValue

		If objBindings.Count > 0 Then
			If Not IsNull(strID) And Len(strID) > 0 Then
				Response.Write "<dl id=""" & strID & """>" & vbCrLf
			Else
				Response.Write "<dl>" & vbCrLf
			End If

			objRE.Pattern = "\s*"

			For Each strName In objBindings.Keys
				strKey = Request.Cookies(LCase(objRE.Replace(strName, "")))
				Response.Write vbTab & "<dt>" & strName & "</dt>" & vbCrLf
				If Len(strKey) > 0 Then
					Response.Write vbTab & "<dd>Access key: <kbd>" & strKey & "</kbd>.</dd>" & vbCrLf
				Else
					Response.write vbTab & "<dd>No access key defined.</dd>" & vbCrLf
				End If

				strValue = objBindings.Item(strName)
				If IsArray(strValue) Then
					Response.Write vbTab & "<dd>Suggested key: <kbd>" & strValue(1) & "</kbd></dd>" & vbCrLf
				End If
			Next
			Response.Write "</dl>" & vbCrLf
		End If
	End Sub
	
	Public Sub generateForm(bXHTML)
		Dim strName, strID

		If objBindings.Count > 0 Then
			objRE.Pattern = "\s*"

			For Each strName In objBindings.Keys
				strID = LCase(objRE.Replace(strName, ""))
				Response.Write "<fieldset>" & vbCrLf
				Response.Write "<legend>Access Key Settings for " & strName & "</legend>" & vbCrLf
				Response.Write "<p>" & vbCrLf
				Response.Write vbTab & "<input type=""checkbox"" name=""" & strID & """ id=""jsak_" & strID & """ value=""" & strID & """"
				If Len(Request.Cookies(strID)) > 0 Then
					Response.Write " checked=""checked"""
				End If
				If bXHTML Then
					Response.Write "/>" & vbCrLf
				Else
					Response.Write ">" & vbCrLf
				End If
				Response.Write vbTab & "<label for=""jsak_" & strID & """>" & strName & "</label>" & vbCrLf
				Response.Write "</p>" & vbCrLf
				Response.Write "<p>" & vbCrLf
				Response.Write vbTab & "<label for=""jsak_" & strID & "binding"">Binding for " & strName & " Key</label>" & vbCrLf
				Response.Write vbTab & "<input type=""text"" size=""4"" value=""" & Request.Cookies(strID) & """ id=""jsak_" & strID & "binding"" name=""" & strID & "binding"""
				If bXHTML Then
					Response.Write "/>" & vbCrLf
				Else
					Response.Write ">" & vbCrLf
				End If
				Response.Write "</p>" & vbCrLf
				Response.Write "</fieldset>" & vbCrLf
			Next
		End If
	End Sub
	
	Public Sub useDefaults()
		Dim strName, arValue

		objRE.Pattern = "\s*"

		For Each strName In objBindings.Keys
			arValue = objBindings.Item(strName)
			If IsArray(arValue) Then
				strName = LCase(objRE.Replace(strName, ""))
				Response.Cookies(strName) = arValue(1)
				Response.Cookies(strName).Expires = dtExpires
			End If
		Next
	End Sub

	Public Function getAccessitem(strItem, bKey, strElement)
		Dim strName, strKey, strValue, strTemp

		objRE.Pattern = "\s*"

		If objBindings.Count > 0 Then
			strName = LCase(objRE.Replace(strItem, ""))
			strKey = Request.Cookies(strName)
			strTemp = objBindings.Item(strItem)

			If IsArray(strTemp) Then
				strValue = strTemp(0)
			Else
				strValue = strTemp
			End If

			If bKey And Len(strKey) = 1 Then
				If strElement = "label" Then
					getAccessitem = "<label for=""" & strValue & """ accesskey=""" & strKey & """>" & strItem & "</label>"
				Else
					getAccessitem = "<a href=""" & strValue & """ accesskey=""" & strKey & """>" & strItem & "</a>"
				End If
			Else
				If strElement = "label" Then
					getAccessitem = "<label for=""" & strValue & """>" & strItem & "</label>"
				Else
					getAccessitem = "<a href=""" & strValue & """>" & strItem & "</a>"
				End If
			End If
		Else
			getAccessitem = ""
		End If
	End Function
End Class
%>